/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PageEventDispatcher.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.page;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DefaultEventDispatcher;
import org.barracudamvc.core.event.EventBroker;
import org.barracudamvc.core.event.EventContext;
import org.barracudamvc.core.event.EventException;


/**
 * The <code>PageEventDispatcher</code> is used to suppress errors that can result
 * from using the page model with session based pages. If a user makes two requests short
 * after each other it can happen that concurrent modifications of the page stored in the
 * session occur. Then for example a control event that expects a certain page to be stored in
 * the session can get into trouble, usually resulting in classcastexceptions.
 * <p>
 * The <code>PageEventDispatcher</code> solves this by synchronizing the request around the
 * session object.
 * <p>
 * Future enhacements might include the possibility to turn the behaviour on or
 * off on a per-session basis.
 *
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public class PageEventDispatcher extends DefaultEventDispatcher {

    /**
     * <code>PAGE_SYNCHRONIZATION_ACTIVATED</code> can be used to globally
     * deactivate the synchronization. Default value is true.
     */
    public static boolean PAGE_SYNCHRONIZATION_ACTIVATED = true;
    private RenderPage _rpe = new RenderPage();

    /**
     * Synchronizes around the session if <code>PAGE_SYNCHRONIZATION_ACTIVATED</code> is true.
     *
     * @param eb an <code>EventBroker</code> value
     * @param context an <code>EventContext</code> value
     * @exception EventException if an error occurs
     */
    public void dispatchEvent(EventBroker eb, EventContext context) throws EventException {
        context.putState(DefaultEventDispatcher.DEFAULT_RESPONSE_EVENT, _rpe);

        if (PAGE_SYNCHRONIZATION_ACTIVATED) {
            HttpServletRequest req = ((ControlEventContext)context).getRequest();
            HttpSession session = req.getSession(false);
            if (session != null && session.getAttribute(PageFactory.PAGE_KEY) != null) {
                Object synchee = new Synchee();
                if (session.getAttribute(PageFactory.SYNCHEE) == null) {
                    session.setAttribute(PageFactory.SYNCHEE, synchee);
                }
                synchronized(synchee) {
                    super.dispatchEvent(eb, context);
                }
                return;
            }
        }
        super.dispatchEvent(eb, context);
    }

    private static class Synchee implements java.io.Serializable {}

}
