/*
 * Copyright (C) 2003 Stefan Armbruster [stefan@armbruster-it.de]
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: XmlFormMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.xmlform;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.barracudamvc.contrib.dbroggisch.repopulation.GroupFormElement;
import org.barracudamvc.contrib.dbroggisch.repopulation.RepopulationFormMap;
import org.barracudamvc.contrib.dbroggisch.repopulation.SelectFormElement;
import org.barracudamvc.contrib.dbroggisch.repopulation.TextAreaFormElement;
import org.barracudamvc.contrib.sam.data.DataObject;
import org.barracudamvc.contrib.sam.data.DataObjectException;
import org.barracudamvc.contrib.sam.data.UploadElement;
import org.barracudamvc.contrib.sam.data.UploadLinkElement;
import org.barracudamvc.contrib.sam.xmlform.dtd.Element;
import org.barracudamvc.contrib.sam.xmlform.dtd.Form;
import org.barracudamvc.contrib.sam.xmlform.dtd.Option;
import org.barracudamvc.contrib.sam.xmlform.dtd.Para;
import org.barracudamvc.contrib.sam.xmlform.dtd.Select;
import org.barracudamvc.contrib.sam.xmlform.dtd.Upload;
import org.barracudamvc.contrib.sam.xmlform.dtd.Validator;
import org.barracudamvc.core.comp.DefaultItemMap;
import org.barracudamvc.core.comp.DefaultListModel;
import org.barracudamvc.core.comp.DefaultListSelectionModel;
import org.barracudamvc.core.comp.ListModel;
import org.barracudamvc.core.comp.ListSelectionModel;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.forms.DefaultFormElement;
import org.barracudamvc.core.forms.FormElement;
import org.barracudamvc.core.forms.FormType;
import org.barracudamvc.core.forms.FormValidator;

/** Provide a FormMap based on a XML description */
public class XmlFormMap extends RepopulationFormMap {

    protected static Logger logger = Logger.getLogger(XmlFormMap.class.getName());

    public static final String GROUP = "GROUP";

    protected String formName;

    protected Form form;

    /**
     * instatiate a new XML Form. First the XML Form description is loaded by calling {@link XmlFormFactory }. For each
     * field, if required a validator is created and the pre-set values are set according the DataObject
     * 
     * @param formName
     *                 filename of the form, used to call {@link XmlFormFactory}
     * @param dataObject
     *                 {@link DataObject}associated with the form
     * @param name
     *                 internal name of the form
     */
    public XmlFormMap(ControlEventContext context, String formName, String name, DataObject dataObject) {
        this.formName = formName;
        setName(name);
        form = XmlFormFactory.get(context, this.formName);
        Iterator iter = form.getElementList().iterator();
        while (iter.hasNext()) {
            Element ele = (Element) iter.next();
            Object o = getFormElements(ele, ele.getName());
            if (o != null) {
                if (o instanceof FormElement) {
                    initFormElement(dataObject, ele, (FormElement) o);
                } else if (o instanceof Collection) {
                    logger.debug("got a collection for " + ele.getName());
                    for (Iterator formElementIter = ((Collection) o).iterator(); formElementIter.hasNext();) {
                        FormElement fe = (FormElement) formElementIter.next();
                        initFormElement(dataObject, ele, fe);
                    }

                } else {
                    logger.error("undefined object class " + o.getClass().getName());
                }
            }
        }
    }

    /**
     * set the orig value of a formelement by retrieving
     * 
     * @param dataObject
     * @param ele
     * @param fe
     * @see DataObject#get(String)
      */
    protected void initFormElement(DataObject dataObject, Element ele, FormElement fe) {
        if (logger.isDebugEnabled()) {
            logger.debug("initializing FormElement " + fe.getName() + " class: " + fe.getClass().getName());
        }
        Object deflt;
        // set default values
        if ((dataObject != null) && !(fe instanceof UploadElement)) {
            try {
                deflt = dataObject.get(ele.getName());
                if (deflt instanceof List) {

                    // if DataObject's get method returns a List (instead of String), we know that this is a multi-value
                    // field
                    List defltList = (List) deflt;
                    SelectFormElement sfe = (SelectFormElement) fe;
                    ListSelectionModel defaults = new DefaultListSelectionModel();
                    defaults.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

                    // the list returned from get is a list of oids. These must be converted in select box indexes
                    ListModel lm = sfe.getListModel();
                    for (int i = 0; i < lm.getSize(); i++) {
                        DefaultItemMap item = (DefaultItemMap) lm.getItemAt(i);
                        if (defltList.contains(item.getKey())) defaults.addSelectionInterval(i, i);
                    }
                    sfe.setListSelectionModel(defaults);

                } else {
                    fe.setOrigVal(deflt);
                    if (logger.isDebugEnabled()) if (deflt instanceof String) {
                        int len = ((String) deflt).length();
                        if (len > 1000) {
                            deflt = "<long data " + len + " bytes>";
                        }

                    }
                    logger.debug("default value for element " + ele.getName() + " is " + deflt);
                }
            } catch (DataObjectException ex) {
                logger.error(ex.getMessage(), ex);
            }
        }
        if (logger.isDebugEnabled()) logger.debug("Adding " + ele.getName() + " to formmap.");
        //fe.setOrigVal(xmlMap.get(ele.getName()));

        fe.setValidator(buildValidatorObject(ele.getValidator()));
        defineElement(fe);

        passIdFieldToUploadElement();

    }

    /**
     * since a
     * 
     * @see UploadLinkElement needs the value of a identity field, a reference is passed here
     */
    protected void passIdFieldToUploadElement() {
        for (Iterator iter = elements.values().iterator(); iter.hasNext();) {
            Object o = iter.next();
            if (o instanceof UploadLinkElement) {
                UploadLinkElement ule = (UploadLinkElement) o;
                String idField = ule.getIdField();
                FormElement idEle = (FormElement) elements.get(idField);
                ule.setIdFormElement(idEle);
            }
        }
    }

    private static Map primitiveClasses = new Hashtable();

    static {
        primitiveClasses.put("int", Integer.TYPE);
        primitiveClasses.put("float", Float.TYPE);
        primitiveClasses.put("double", Double.TYPE);
        primitiveClasses.put("long", Long.TYPE);
        primitiveClasses.put("short", Short.TYPE);
        primitiveClasses.put("boolean", Boolean.TYPE);
        primitiveClasses.put("byte", Byte.TYPE);
        primitiveClasses.put("char", Character.TYPE);
    }

    private Class getClassWithPrimitive(String name) throws ClassNotFoundException {
        Class c = (Class) primitiveClasses.get(name);
        if (c == null) c = Class.forName(name);
        return c;
    }

    private Object initialize(Class c, String value) throws NoSuchMethodException, InstantiationException,
            IllegalAccessException, InvocationTargetException {
        Class[] types = new Class[1];
        types[0] = String.class;
        Object[] para = new Object[1];
        para[0] = value;
        Constructor con = c.getConstructor(types);
        return con.newInstance(para);
    }

    /** recursivly called to construct a validator hierarchy. Internally, the reflection API is used here */
    protected FormValidator buildValidatorObject(Validator validator) {
        if (validator == null) return null;

        List children = new Vector();

        //if (validator.getValidatorList() != null) {
        Iterator iter = validator.getValidatorList().iterator();
        while (iter.hasNext()) {
            Validator v = (Validator) iter.next();
            // get sub-validtors recursively
            children.add(buildValidatorObject(v));
        }
        //}
        try {
            // use reflection to instanciate validator
            logger.info("Trying to create Validator " + validator.getClassname());
            Class validatorClass = Class.forName(validator.getClassname());
            if (logger.isDebugEnabled()) logger.debug("Class successfully found");

            // find matching constructor, first the childvalidators and second the specific parameters are appendendA
            List para = validator.getParaList();

            int size = children.size() + para.size();
            if (logger.isDebugEnabled()) logger.debug("constructor will have " + size + " arguments");
            Class[] types = new Class[size];
            Object[] values = new Object[size];
            int counter = 0;
            // insert constructor parameter from children formvalidators
            iter = children.iterator();
            while (iter.hasNext()) {
                Object o = iter.next();
                types[counter] = FormValidator.class;
                values[counter++] = o;
            }
            // insert construcor parameter from <para> elements
            iter = para.iterator();
            while (iter.hasNext()) {
                Para p = (Para) iter.next();
                types[counter] = getClassWithPrimitive(p.getClassname());
                values[counter] = initialize(types[counter], p.getValue());
                counter++;
                if (logger.isDebugEnabled()) logger.debug("counter is " + counter);
            }

            Constructor c = validatorClass.getConstructor(types);
            if (logger.isDebugEnabled()) logger.debug("Constructor with " + children.size() + " arguments found");
            // invoke the constructor
            return (FormValidator) c.newInstance(values);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            return null;
        }
    }

    /** generate a FormElement for a given Element from the XML description */
    protected Object getFormElements(Element ele, String name) {
        FormElement fe = null;
        if (ele.getText() != null)
            fe = new DefaultFormElement(name, FormType.STRING, null);
        else if (ele.getUpload() != null) {
            // for an Upload, we need to FormElements:
            // 1) UploadElements represents the respective Input element
            // 2) UploadLinkElement represents a link to previosly existing upload file
            Upload u = ele.getUpload();
            Collection c = new ArrayList();
            c.add(new UploadElement(name));
            c.add(new UploadLinkElement(name + XmlFormViewHandler.UPLOAD_LINK_POSTFIX, name, u.getContent_type_field(),
                    u.getFile_name_field(), u.getLink_event(), u.getId_field()));
            return c;
        } else if (ele.getTextarea() != null) {
            fe = new TextAreaFormElement(name, FormType.STRING, null);
            logger.debug("creating a textarea element" + name);
        } else if (ele.getSelect() != null) {
            Select sel = ele.getSelect();
            DefaultListModel lm = new DefaultListModel();
            // call RMI to get select box contents
            if (sel.getXmlclass() != null) {
                logger.debug("class is " + sel.getXmlclass());
                try {
                    Class c = Class.forName(sel.getXmlclass());
                    Class[] params = new Class[0];
                    Object[] args = new Object[0];
                    Method method = c.getMethod(sel.getMethod(), params);
                    Object result = method.invoke(null, args);
                    logger.debug("result is " + result.getClass().getName());
                    if (result instanceof List) {
                        List l = (List) result;
                        Iterator rmiIter = l.iterator();
                        while (rmiIter.hasNext())
                            lm.add(rmiIter.next());
                    }
                } catch (Exception e) {
                    logger.error(e.getMessage(), e);
                }
            }
            Iterator subIter = ele.getSelect().getOptionList().iterator();
            while (subIter.hasNext()) {
                Option opt = (Option) subIter.next();
                lm.add(new DefaultItemMap(opt.getValue2(), opt.getValue()));
            }
            //fe = new SelectFormElement(lm, name, FormType.STRING, null, new NotNullValidator("Bitte w�hlen Sie im
            // Feld " + ele.getLabel() + " eine Alternative aus."), false);
            boolean hasMulti = false;
            String m = sel.getMultivalue();
            if ((m != null) && (m.equals("true") || m.equals("yes"))) hasMulti = true;
            fe = new SelectFormElement(lm, name, FormType.STRING, null, null, hasMulti);
        } else if (ele.getHidden() != null) {
            fe = new DefaultFormElement(name, FormType.STRING, null);
        } else if (ele.getCheckbox() != null) {
            DefaultListModel lm = new DefaultListModel();
            Option o;
            for (Iterator iter = ele.getCheckbox().getOptionList().iterator(); iter.hasNext(); lm
                    .add(new DefaultItemMap(o.getValue2(), o.getLabel())))
                o = (Option) iter.next();

            fe = new GroupFormElement(lm, name, FormType.STRING, "false", null, true);
        } else if (ele.getRadio() != null) {
            DefaultListModel lm = new DefaultListModel();
            Option o;
            for (Iterator iter = ele.getCheckbox().getOptionList().iterator(); iter.hasNext(); lm
                    .add(new DefaultItemMap(o.getValue2(), o.getLabel())))
                o = (Option) iter.next();

            fe = new GroupFormElement(lm, name, FormType.STRING, null, null, false);
        }
        return fe;
    }

    public Form getForm() {
        return form;
    }

    private boolean first = true;

    /*
     * (non-Javadoc)
     * 
     * @see org.barracudamvc.contrib.dbroggisch.repopulation.RepopulationFormMap#getItem(java.lang.String)
     */
    public Object getItem(String key) {
        Object v = super.getItem(key);
        logger.debug("getItem: Key: " + key + ", value " + v);
        if (first) {
            first = false;
            for (Iterator iter = elements.entrySet().iterator(); iter.hasNext();) {
                Map.Entry entry = (Map.Entry) iter.next();
                logger.debug(entry.getKey() + " -> " + entry.getValue().getClass().getName());
            }
        }
        return v;
    }

}