/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DefaultFormMapper.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms;

import java.util.*;

import org.apache.log4j.Logger;
import org.barracudamvc.plankton.data.*;

/**
 * <p>A FormMapFilter is used to provide a control what elements
 * get mapped in a form
 *
 * @author  Christian Cryder [christianc@granitepeaks.com]
 * @since   2.0
 */
public class DefaultFormMapper implements FormMapper {

    //public constants
    protected static final Logger localLogger = Logger.getLogger(DefaultFormMapper.class.getName());

    //working vars
    protected Map mappedElements = new TreeMap();
    protected boolean iterateOverParams = false;

    //--------------- FormMapper ---------------------------------
    /**
     * Map a whole form
     */
    public FormMap mapForm(FormMap fm, Map paramMap) {
        if (localLogger.isInfoEnabled()) localLogger.info("FormMapper "+this+".mapForm() - start"); 
        if (localLogger.isDebugEnabled()) {
            CollectionsUtil.printStackTrace(paramMap);
        }
        
        //let the mapper know we're ready to start
        this.preMap();

        //now iterate through the set (note the default mapping approach is to 
        //iterate through the paramSet and lookup form elements. An alternate 
        //strategy would be to implement through defined form elements and lookup params
        Iterator it = (iterateOverParams ? paramMap.entrySet().iterator() : fm.getElements().entrySet().iterator());
        while (it.hasNext()) {
            //get the next entry in the source data structure
            Map.Entry me = (Map.Entry) it.next();
            String key = (String) me.getKey();
            Object o = (iterateOverParams ? me.getValue() : paramMap.get(key));
        
            //get the filter associated with the key
            FormElement el = this.getElementForMapping(fm, key);
            if (localLogger.isDebugEnabled()) localLogger.debug("...param key:"+key+" param val:"+o+" {"+(o!=null ? o.getClass().getName() : "")+"} "+(el!=null ? ""+el : "-no match-"));
            if (el==null) continue;

            //now map the values. If there are multiple values for
            //a given key, the values will be mapped into List structures
            Object origVal = null;
            Object val = null;
            if (o instanceof List) o = ((List) o).toArray();
            if (o instanceof Object[]) {
                Object origVals[] = (Object[]) o;
                int max = origVals.length;
                //if there's only one
                if (max==1) {
                    origVal = origVals[0];
                    val = _mapElement(fm, el, origVal);
                //if there's multiple values
                } else {
                    origVal = new ArrayList();
                    val = new ArrayList();
                    List lorigVal = (List) origVal;
                    List lval = (List) val;
                    for (int i=0; i<max; i++) {
                        lorigVal.add(origVals[i]);
                        lval.add(_mapElement(fm, el, origVals[i]));
                    }
                 }
            } else {
                origVal = o;
                val = _mapElement(fm, el, origVal);
            }

            //place the values in the form element
            el.setOrigVal(origVal);
            el.setVal(val);

            //place the element back into the form map (this is important,
            //since the filter may return a different element in certain 
            //situations)
            fm.defineElement(el);
            mappedElements.put(el.getKey(), el);
        }
        
        //let the mapper know we're done mapping
        this.postMap();
        
        //return the reference to the form map
        if (localLogger.isInfoEnabled()) localLogger.info("FormMapper "+this+".mapForm() - finished"); 
        return fm;
    }
    
    /**
     * Map an individual element
     */
    public FormElement mapElement(FormMap fm, String key, Object origVal) {
        if (localLogger.isInfoEnabled()) localLogger.info("FormMapper "+this+".mapElement() - start"); 

        //default behavior is that an individual element will only be mapped if its already defined
        FormElement el = fm.getElement(key);
        if (el==null) return null;
        
        //we delegate to the mapForm() function here to make sure this method of mapping 
        //also makes use of the filter stuff, as well as handling Object[] vals, etc
        boolean orig = iterateOverParams;
        iterateOverParams = true;
        Map map = new TreeMap();
        map.put(key, origVal);
        mapForm(fm, map);
        if (localLogger.isInfoEnabled()) localLogger.info("FormMapper "+this+".mapElement() - finished"); 
        iterateOverParams = orig;
        return el;
    }

    /**
     * Actually map a value to a form element. Returns the mapped value.
     */
    protected Object _mapElement(FormMap fm, FormElement element, Object origVal) {
        if (localLogger.isInfoEnabled()) localLogger.info("Mapping Element: "+element.getKey()+"="+origVal);
        Object val = null;
        FormType type = element.getType();

        // dbr_20030504 Instead of null-testing, we now use the
        // customizable isNull-function. In its default-implementation,
        // it will also consider "" and whitespaces beeing null. This
        // prevents possible type-conflicts discovered by Shawn Wilson.
        if (!this.isNull(origVal)) {
            //parse the value; catch any exceptions and use the default
            //value instead
            try {
                // don't need to parse if origVal is the expected type
                Class typeclass = type.getFormClass();
                if (typeclass.isInstance(origVal)) val = origVal;
                else val = type.parse(origVal.toString(), fm.getLocale());
            } catch (ParseException e) {
                if (localLogger.isDebugEnabled()) localLogger.debug("ParseException:", e);
                element.setParseException(e);
            }
        }

        if (val==null) {
            if (localLogger.isDebugEnabled()) localLogger.debug("Using default val");
            val = element.getDefaultVal();
        }

        if (localLogger.isDebugEnabled()) localLogger.debug("Result: "+val);
        return val;
    }

    /**
     * Returns a map containing all elements mapped by this mapper (will return an empty
     * map if the form has not been mapped yet)
     */
    public Map getElements() {
        return mappedElements;
    }



    //------------------------------------------------------------
    //protected methods for custom mappers to override
    //------------------------------------------------------------
    /**
     * Invoked before mapping begins
     */
    protected void preMap() {
        //nop
    }

    /**
     * This method is used by the mapping process to look up FormElements. It
     * basically provides a way for the filter to determine whether or not
     * mapping should occur. You can override this method of control what gets
     * mapped and what doesn't. Based on the target key, the mapper either passes 
     * back the appropriate FormElement to map the key to, or it returns null to
     * indicate the element should not be mapped.
     */
    protected FormElement getElementForMapping(FormMap fm, String paramKey) {
        return fm.getElement(paramKey);
    }

    /**
     * Invoked after mapping is completed
     */
    protected void postMap() {
        //nop
    }
    
    /**
     * This method defines what the FormMap identifies as null-value. If this
     * doesn't suit your needs, you can simply overload it.
     *
     * @param val The value from request or statemap
     * @return if the value is considered to be null
     */
    protected boolean isNull(Object val) {
        return ((val==null) || (val.toString().trim().length() < 1));
    }

}
