/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LoginScreen.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.examples.ex1;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BAction;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BInput;
import org.barracudamvc.core.comp.BScript;
import org.barracudamvc.core.comp.BSelect;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.BaseEventListener;
import org.barracudamvc.core.event.ClientSideRedirectException;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.DefaultListenerFactory;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.InterruptDispatchException;
import org.barracudamvc.core.event.ListenerFactory;
import org.barracudamvc.core.event.helper.BTemplateViewHandler;
import org.barracudamvc.core.event.helper.EventForwardingFactory;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.core.util.http.SessionServices;
import org.barracudamvc.examples.ex1.events.AttemptLogin;
import org.barracudamvc.examples.ex1.events.BadPassword;
import org.barracudamvc.examples.ex1.events.BadUsername;
import org.barracudamvc.examples.ex1.events.ErrorEvent;
import org.barracudamvc.examples.ex1.events.GetLoginScreen;
import org.barracudamvc.examples.ex1.events.GetMainScreen;
import org.barracudamvc.examples.ex1.events.RenderLoginScreen;
import org.barracudamvc.examples.ex1.events.UnknownLoginError;
import org.barracudamvc.examples.ex1.xmlc.BadPasswordHTML;
import org.barracudamvc.examples.ex1.xmlc.BadUsernameHTML;
import org.barracudamvc.examples.ex1.xmlc.ErrorHTML;
import org.barracudamvc.examples.ex1.xmlc.LoginHTML;
import org.barracudamvc.examples.ex1.xmlc.LoginSnippetHTML;


/**
 * Event handlers (both Controller and View) for the 
 * Login screen
 */
public class LoginScreen extends DefaultEventGateway {

    //public constants
    protected static final Logger logger = Logger.getLogger(LoginScreen.class.getName());

    //this defines the various event handlers
    private ListenerFactory getLoginFactory =               new EventForwardingFactory(new RenderLoginScreen());
    private ListenerFactory attemptLoginFactory =           new DefaultListenerFactory() {public BaseEventListener getInstance() {return new AttemptLoginHandler();}        public String getListenerID() {return getID(AttemptLoginHandler.class);}};                
    private ListenerFactory renderLoginFactory =            new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderLoginScreenHandler();}    public String getListenerID() {return getID(RenderLoginScreenHandler.class);}};                
    private ListenerFactory renderBadUsernameFactory =      new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderBadUsernameHandler();}    public String getListenerID() {return getID(RenderBadUsernameHandler.class);}};                
    private ListenerFactory renderBadPasswordFactory =      new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderBadPasswordHandler();}    public String getListenerID() {return getID(RenderBadPasswordHandler.class);}};                
    private ListenerFactory renderErrorFactory =            new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderErrorHandler();}            public String getListenerID() {return getID(RenderErrorHandler.class);}};

    /**
     * Public constructor
     */
    public LoginScreen() {
        //specify who's interested in what
        specifyLocalEventInterests(getLoginFactory, GetLoginScreen.class);
        specifyLocalEventInterests(attemptLoginFactory, AttemptLogin.class);
        specifyLocalEventInterests(renderLoginFactory, RenderLoginScreen.class);
        specifyLocalEventInterests(renderBadUsernameFactory, BadUsername.class);
        specifyLocalEventInterests(renderBadPasswordFactory, BadPassword.class);
        specifyLocalEventInterests(renderErrorFactory, ErrorEvent.class);
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * AttemptLoginHandler - this is where we handle any AttemptLogin 
     * events. Validate and then redirect accordingly.
     */
    class AttemptLoginHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletRequest req = context.getRequest();
            logger.debug("Attempting login...");

            try {
                //validate the login information
                int valid = LoginServices.validateUserPwd(req, true);
                logger.debug("Validity:"+LoginServices.xrefValidity(valid));

                //if it's valid, redirect to the Main screen
                if (valid==LoginServices.VALID) {
                    logger.debug("...Redirecting to Main Screen");
                    throw new ClientSideRedirectException(new GetMainScreen());
                //invalid user                
                } else if (valid==LoginServices.INVALID_USER) {
                    logger.debug("...Invalid User!");
                    throw new InterruptDispatchException(new BadUsername());
                //invalid pwd                
                } else if (valid==LoginServices.INVALID_PASSWORD) {
                    logger.debug("...Invalid Password!");
                    throw new InterruptDispatchException(new BadPassword());
                //anything else
                } else {
                    logger.debug("...Unknown Error!");
                    throw new InterruptDispatchException(new UnknownLoginError());
                }
            } finally {
                event.setHandled(true);
            }
        }
    }



    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderLoginScreenHandler - this is where we handle any RenderLoginScreen 
     * event and actually generate the view. We repoulate the form if there
     * is user/pwd information in the session.
     */
    class RenderLoginScreenHandler extends BTemplateViewHandler {
        public Object getTemplateModels() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return LoginHTML.class;}
    }

    /**
     * RenderBadUsernameHandler - this is where we handle any RenderBadUsername
     * event and actually generate the error view. We could just redisplay the
     * Login screen if we wanted and add an error message there, but I wanted to
     * demonstrate how we can throw InterruptDispatchExceptions to redirect
     * flow here.
     */
    class RenderBadUsernameHandler extends BTemplateViewHandler {
        public Object getTemplateModels() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return BadUsernameHTML.class;}
    }

    /**
     * RenderBadPasswordHandler - this is where we handle any RenderBadUsername
     * event and actually generate the error view. We could just redisplay the
     * Login screen if we wanted and add an error message there, but I wanted to
     * demonstrate how we can throw InterruptDispatchExceptions to redirect
     * flow here.
     */
    class RenderBadPasswordHandler extends BTemplateViewHandler {
        public Object getTemplateModels() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return BadPasswordHTML.class;}
    }

    /**
     * RenderErrorHandler - this is where we handle any Error event and generate a 
     * default view. Typically, this would only happen when a specific error was 
     * not caught and handled. In that case, the event proprogates up the event
     * chain, and we catch it here.
     */
    class RenderErrorHandler extends BTemplateViewHandler {
        public Object getTemplateModels() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return ErrorHTML.class;}
    }

    //------------------------------------------------------------
    //                Components - TemplateModel
    //------------------------------------------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
    
        //register the model by name
        public String getName() {return "Ex1_Login";}
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("GetLoginLink")) {
                return new BAction(new GetLoginScreen());

            } else if (key.equals("AttemptLoginLink")) {
                return new BAction(new AttemptLogin());
//                BAction ba = new BAction(new AttemptLogin());
//                ba.addChild(new BScript(BScript.ON_CLICK, "alert('Hi Ma!');", BScript.PREPEND, "scripts/foo.js"));
//                ba.addChild(new BScript(BScript.ON_CLICK, "alert('Hi Pa!');", BScript.APPEND, "scripts/foo.js"));
//                return ba;

            } else if (key.equals("ResetLoginLink")) {
//System.out.println("Here 0");
                BComponent bcomp = new BComponent();
                bcomp.setAttr("title", "click here to reset form");
                return bcomp;

            } else if (key.equals("TestSelect")) {
                BSelect bs = new BSelect();
                bs.setAction(new BAction(new GetLoginScreen()));
                bs.addChild(new BScript(BScript.ON_CHANGE, "alert('Hi Ma!');", BScript.PREPEND, "scripts/foo.js"));
                bs.addChild(new BScript(BScript.ON_CHANGE, "alert('Hi Pa!');", BScript.APPEND, "scripts/foo.js"));
                return bs;
                
            } else if (key.equals("TestRow")) {
                try {
                    LoginSnippetHTML page = (LoginSnippetHTML) DefaultDOMLoader.getGlobalInstance().getDOM(LoginSnippetHTML.class);
                    return page.getElementLoginSnippet();
                } catch (IOException e) {
                    return null; 
                }
/*
                Element elroot = (Element) vc.getDocument().createElement("SPAN");
                Element el = (Element) vc.getDocument().createElement("SELECT");
                el.setAttribute("class", "Dir::Get_Data.Ex1_Login.TestSelect");
                el.setAttribute("foo", "blah");
                elroot.appendChild(el);
                return elroot;
*/                
            } else if (key.equals("User")) {
                HttpSession session = SessionServices.getSession(vc, false);
                String user = (String) session.getAttribute(LoginServices.USER);
                if (user==null) user = "";
                return new BInput(null, LoginServices.USER, user);

            } else if (key.equals("Password")) {
                HttpSession session = SessionServices.getSession(vc, false);
                String password = (String) session.getAttribute(LoginServices.PASSWORD);
                if (password==null) password = "";
                return new BInput(null, LoginServices.PASSWORD, password);

            } else {
                return super.getItem(key);
            }
        }
    }
}

