package org.barracudamvc.testbed.workbench;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.UIDefaults;
import javax.swing.UIManager;

import org.barracudamvc.plankton.data.ObjectRepositoryAssembler;
import org.barracudamvc.testbed.workbench.jdbc.JDBCTester;
import org.barracudamvc.testbed.workbench.stress.URLStressTester;

/**
 * This class provides the JPanel that acts as the WorkBench. You should
 * run this class directly to launch the Workbench GUI.
 *
 * It contains a JTabbedPane which in turn contains an instance of 
 * various workbench utilities. You should refer to those files for 
 * full documentation.
 *
 * The basic purpose of this class (and subpackages) is to provide a nice
 * place to add gui utilities that we write to aid in debugging Barracuda
 * and DB webapps. Right now there are 2 distinct components: a URL Stress 
 * tester utility (allows you to stress test a simple URL), and a JDBC DB SQL 
 * utility (allows you to connect to a DB via JDBC and execute SQL commands).
 */
public class WorkBench extends JPanel {

    //debug vars
    public static int showDebug = 2;

    //objects

    
    /**
     * Public constructor
     */
    public WorkBench() {
        //init the screen and catch any unexpected errors
        try {
            initViews();
        } catch (Exception be) {
            System.out.println ("Error initializing "+this);
            be.printStackTrace();
        }
    }
    
    /**
     * This method is responsible for laying out the
     * UI components. 
     *
     * @throws Exception
     */
    private void initViews() throws Exception {
        //basic setup
        this.setLayout(new BorderLayout());

        //create a tab pane
        JTabbedPane jtpTabs = new JTabbedPane();
        this.add(jtpTabs, BorderLayout.CENTER);
        
        //...add the URLStressTester
        URLStressTester urlStressTester = new URLStressTester();
        jtpTabs.addTab("URL Stress Tester", null, urlStressTester, "Perform URL Stress Testing");
        
        //...add the JDBCTester
        JDBCTester jdbcTester = new JDBCTester();
        jtpTabs.addTab("JDBC Tester", null, jdbcTester, "Connect to a JDBC Datasource");
        
        //...add the ??? example
        //(if you want to add something to the workbench, this is the place to do it)
    }


    public static void main(String[] args) {
        //first and foremost, setup the object repository
        String assemblerFile = "../object-repository-workbench.xml";
        try {
            new ObjectRepositoryAssembler().assemble(assemblerFile);
        } catch (Exception e) {
            System.out.println("Unexpected exception trying to assemble "+assemblerFile+":"+e);
            System.out.println("This is not a fatal err, but you may need to run ant install first to create it.");
        }
        
        //create the master frame
        JFrame frame = new JFrame();
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent event) {
                System.exit(0);    // close the application
            }
        });

        //add in the WorkBench
        WorkBench masterView = new WorkBench();
        frame.getContentPane().add(masterView, BorderLayout.CENTER);

        //finally, size and show        
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setTitle("Barracuda Workbench");
        frame.setSize(new Dimension(800, 600));
        frame.setLocation((d.width-frame.getSize().width)/2, (d.height-frame.getSize().height)/2);
        frame.setVisible(true);
    }

    /**
     * static initializer for look & feel adjustments
     */
    static {
        try {
            //set the look & feel and then get the UIDefaults
            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
            UIDefaults uid = UIManager.getLookAndFeelDefaults();
            
            //override the default CheckBox foreground to be the same color as JLabels
            uid.put("CheckBox.foreground", uid.get("Label.foreground"));
            uid.put("RadioButton.foreground", uid.get("Label.foreground"));
        } catch (Exception e) {}
    }
    

}
